Index: src/lib/codecs/speex/filters/dsfSpeexEncoder/SpeexEncodeInputPin.cpp
===================================================================
--- src/lib/codecs/speex/filters/dsfSpeexEncoder/SpeexEncodeInputPin.cpp	(rvision 11936)
+++ src/lib/codecs/speex/filters/dsfSpeexEncoder/SpeexEncodeInputPin.cpp	(copie de travail)
@@ -32,6 +32,8 @@
 #include "stdafx.h"
 #include "SpeexEncodeInputPin.h"
 
+#define ADAPT_FRAME_RATE 1
+
 SpeexEncodeInputPin::SpeexEncodeInputPin(       AbstractTransformFilter* inParentFilter
                                             ,   CCritSec* inFilterLock
                                             ,   AbstractTransformOutputPin* inOutputPin
@@ -80,8 +82,28 @@
     unsigned long locNumSamplesPerChannel = bufferBytesToSampleCount(inNumBytes);
     locPackets = mSpeexEncoder.encode((const short* const)inBuf, locNumSamplesPerChannel);
 
+#ifdef ADAPT_FRAME_RATE
+	/* skip packet if we are too late ? */
+	__int64 curTime = (mDsTimeStart*mEncoderSettings.sampleRate())/1000;
+	if (mUptoFrame-curTime > locNumSamplesPerChannel) {
+		return S_OK;
+	}
+#endif
+
     locHR = sendPackets(locPackets);
     deletePacketsAndEmptyVector(locPackets);
+
+#ifdef ADAPT_FRAME_RATE
+	/* Resend same packet if we are too early */
+	if (!FAILED(locHR)) {
+		while (curTime-mUptoFrame > locNumSamplesPerChannel) {
+		    locPackets = mSpeexEncoder.encode((const short* const)inBuf, locNumSamplesPerChannel);
+			locHR = sendPackets(locPackets);
+			deletePacketsAndEmptyVector(locPackets);
+		}
+	}
+#endif
+
     return locHR;
 
 }
Index: src/lib/codecs/theora/filters/dsfTheoraEncoder/TheoraEncodeInputPin.cpp
===================================================================
--- src/lib/codecs/theora/filters/dsfTheoraEncoder/TheoraEncodeInputPin.cpp	(rvision 11936)
+++ src/lib/codecs/theora/filters/dsfTheoraEncoder/TheoraEncodeInputPin.cpp	(copie de travail)
@@ -32,6 +32,9 @@
 #include "stdafx.h"
 #include "theoraencodeinputpin.h"
 
+/* Define to adapt theora frame rate to DirectShow clock */
+#define ADAPT_FRAME_RATE 1
+
 TheoraEncodeInputPin::TheoraEncodeInputPin(AbstractTransformFilter* inParentFilter, CCritSec* inFilterLock, AbstractTransformOutputPin* inOutputPin, vector<CMediaType*> inAcceptableMediaTypes)
 	:	AbstractTransformInputPin(inParentFilter, inFilterLock, inOutputPin, NAME("TheoraEncodeInputPin"), L"YV12 In", inAcceptableMediaTypes)
 	,	mXOffset(0)
@@ -43,6 +46,7 @@
 	,	mVideoFormat(NULL)
 	
 
+	, numFrames(0)
 {
 	//debugLog.open("g:\\logs\\theoencfiltinput.log", ios_base::out);
 	mYUV.y = NULL;
@@ -1237,7 +1241,17 @@
 		throw 0;
 	}
 	
+#ifdef ADAPT_FRAME_RATE
+	__int64 frameTime = (1000 * mTheoraInfo.fps_denominator) / mTheoraInfo.fps_numerator;
+	__int64 curTheoraTime = (numFrames * 1000 * mTheoraInfo.fps_denominator) / mTheoraInfo.fps_numerator;
 
+	// Skip a frame if we are too late
+	if (curTheoraTime-mDsTimeStart > frameTime) {
+		//debugLog<<" too late: "<< curTheoraTime << "," << timeStart << ":" << frameTime <<endl;
+		return S_OK;
+	}
+#endif
+
 	StampedOggPacket* locPacket = mTheoraEncoder.encodeTheora(&mYUV);
 	if (locPacket == NULL) {
 		//debugLog<<"Encode returns NULL"<<endl;
@@ -1250,9 +1264,51 @@
 	//We still own the packet after this, we have to delete it.
 	locHR = deliverData(locFrameStart, locFrameEnd, locPacket->packetData(), locPacket->packetSize());
 	delete locPacket;
+
+#ifdef ADAPT_FRAME_RATE
+	if (!FAILED(locHR)) {
+		++numFrames;
+		locHR = encodeMoreFrames();
+	}
+#endif
+
 	return locHR;
+}
 
+HRESULT TheoraEncodeInputPin::encodeMoreFrames(void)
+{
+	HRESULT locHR = S_OK;
+
+	__int64 frameTime = (1000 * mTheoraInfo.fps_denominator) / mTheoraInfo.fps_numerator;
+	__int64 curTheoraTime = (numFrames * 1000 * mTheoraInfo.fps_denominator) / mTheoraInfo.fps_numerator;
+
+	/* Resend previous packet, if we are too fast */
+	while (mDsTimeStart-curTheoraTime > frameTime) {
+		LONGLONG locFrameStart = mUptoFrame;
+		LONGLONG locFrameEnd = 0;
+
+		StampedOggPacket* locPacket = mTheoraEncoder.encodeTheora(&mYUV);
+		if (locPacket == NULL) {
+			//debugLog<<"Encode returns NULL"<<endl;
+			return S_FALSE;
+		}
+		locFrameEnd		= mUptoFrame = locPacket->endTime();
+		//debugLog<<"Delivering..."<<endl;
+
+		//We still own the packet after this, we have to delete it.
+		locHR = deliverData(locFrameStart, locFrameEnd, locPacket->packetData(), locPacket->packetSize());
+		delete locPacket;
+		if (FAILED(locHR)) {
+			break;
+		}
+
+		++numFrames;
+		curTheoraTime = (numFrames * 1000 * mTheoraInfo.fps_denominator) / mTheoraInfo.fps_numerator;
+	}
+
+	return locHR;
 }
+
 bool TheoraEncodeInputPin::ConstructCodec() {
 
 	//debugLog<<"Contructing codec..."<<endl;
Index: src/lib/codecs/theora/filters/dsfTheoraEncoder/TheoraEncodeInputPin.h
===================================================================
--- src/lib/codecs/theora/filters/dsfTheoraEncoder/TheoraEncodeInputPin.h	(rvision 11936)
+++ src/lib/codecs/theora/filters/dsfTheoraEncoder/TheoraEncodeInputPin.h	(copie de travail)
@@ -111,5 +111,8 @@
 	//fstream debugLog;
 	//

-	
+
+	/* To adapt theora frame rate to variable directshow clock */
+	unsigned __int64 numFrames;
+	HRESULT encodeMoreFrames(void);
 };
Index: src/lib/codecs/vorbis/filters/dsfVorbisEncoder/VorbisEncodeInputPin.cpp
===================================================================
--- src/lib/codecs/vorbis/filters/dsfVorbisEncoder/VorbisEncodeInputPin.cpp	(rvision 11936)
+++ src/lib/codecs/vorbis/filters/dsfVorbisEncoder/VorbisEncodeInputPin.cpp	(copie de travail)
@@ -32,6 +32,8 @@
 #include "stdafx.h"
 #include "VorbisEncodeInputPin.h"
 
+#define ADAPT_FRAME_RATE 1
+
 VorbisEncodeInputPin::VorbisEncodeInputPin(     AbstractTransformFilter* inParentFilter
                                             ,   CCritSec* inFilterLock
                                             ,   AbstractTransformOutputPin* inOutputPin
@@ -80,10 +82,29 @@
     unsigned long locNumSamplesPerChannel = bufferBytesToSampleCount(inNumBytes);
     locPackets = mVorbisEncoder.encodeVorbis((const short* const)inBuf, locNumSamplesPerChannel);
 
+#ifdef ADAPT_FRAME_RATE
+	/* skip packet if we are too late ? */
+	__int64 curTime = (mDsTimeStart*mEncoderSettings.mSampleRate)/1000;
+	if (mUptoFrame-curTime > locNumSamplesPerChannel) {
+		return S_OK;
+	}
+#endif
+
     locHR = sendPackets(locPackets);
     deletePacketsAndEmptyVector(locPackets);
-    return locHR;
 
+#ifdef ADAPT_FRAME_RATE
+	/* Resend same packet if we are too early */
+	if (!FAILED(locHR)) {
+		while (curTime-mUptoFrame > locNumSamplesPerChannel) {
+		    locPackets = mVorbisEncoder.encodeVorbis((const short* const)inBuf, locNumSamplesPerChannel);
+			locHR = sendPackets(locPackets);
+			deletePacketsAndEmptyVector(locPackets);
+		}
+	}
+#endif
+
+	return locHR;
 }
 
 bool VorbisEncodeInputPin::ConstructCodec() 
Index: src/lib/core/directshow/libDirectshowAbstracts/AbstractTransformInputPin.cpp
===================================================================
--- src/lib/core/directshow/libDirectshowAbstracts/AbstractTransformInputPin.cpp	(rvision 11936)
+++ src/lib/core/directshow/libDirectshowAbstracts/AbstractTransformInputPin.cpp	(copie de travail)
@@ -41,7 +41,7 @@
 	
 	,	mAcceptableMediaTypes(inAcceptableMediaTypes)
 
-
+	,	mDsTimeStart(0), mDsTimeEnd(0)
 {
 	mStreamLock = new CCritSec;			//Deleted in destructor.
 }
@@ -113,6 +113,14 @@
 		} else {
             //http://windowssdk.msdn.microsoft.com/en-us/library/ms787541.aspx
             //Consider using receive to validate conditions ^^^
+
+			/* Read DirectShow timestamps */
+			REFERENCE_TIME pTimeStart, pTimeEnd;
+			if (inSample->GetTime(&pTimeStart, &pTimeEnd)==NOERROR) {
+				mDsTimeStart = pTimeStart/10000;
+				mDsTimeEnd = pTimeEnd/10000;
+			}
+
 			HRESULT locResult = TransformData(locBuff, inSample->GetActualDataLength());
 			if (locResult == S_OK) {
 				return S_OK;
Index: src/lib/core/directshow/libDirectshowAbstracts/AbstractTransformInputPin.h
===================================================================
--- src/lib/core/directshow/libDirectshowAbstracts/AbstractTransformInputPin.h	(rvision 11936)
+++ src/lib/core/directshow/libDirectshowAbstracts/AbstractTransformInputPin.h	(copie de travail)
@@ -97,6 +97,9 @@
 
 	HRESULT mHR;
 
+	//DirectShow timestamps in milliseconds
+	__int64 mDsTimeStart, mDsTimeEnd;
+
 	//TODO::: Couldn't this be gotten from the parent filter if we make them friends ??
 	AbstractTransformOutputPin* mOutputPin;
 	AbstractTransformFilter* mParentFilter;